// basket.cpp
/***************************************************************************
This program will read a file containing correctly formatted information
from one of your high school's varsity or JV teams:
  "girls.txt", "boys.txt", "girlsjv.txt", or "boysjv.txt".
If the file was successfully read, the roster of the team is printed
in order by player number.  The user is then asked to choose a starting
team.  The program will determine if the starting team is valid.
	Barbara Larson TJHSST 8/24/89
	Modified 12/21/93 - 12/20/94 - 7/26/95
	Translated into C++ by Mr. Samson 12/11/97
	Modified 1/11/99
***************************************************************************/
#include <fstream.h>  // external textfile and cin, cout
#include <conio.h>    // getch()
#include <stdlib.h>   // exit()
#include <iomanip.h>  // setwidth()
#include "apstring.h"
#include "apvector.h"

const apstring error   = "Not a valid player.  Try again. ";
const int NumJerseys   = 56;                // Maximum number of jerseys
const int NumberOnTeam = 5;                 // Maximum players on team

enum PositionType {center,forward,guard};   // possible positions
struct PlayerType
{
    apstring name;
    PositionType position;
    int height;
    bool exists;                        // true if player has that number
};

typedef apvector<PlayerType> TeamType;	 //(NumJerseys);
typedef apvector<int> StartingType;	    //(NumberOnTeam);
typedef apvector<int> RosterType;	    //(PositionType(3));

//Function Prototypes
void Initialize(TeamType& team);
void ReadFile(TeamType& team);
void PrintHeadings(bool All);
void PrintPlayer(int number, PlayerType& player , bool All);
bool Duplicates(StartingType& StartingTeam);
bool Valid(StartingType& StartingTeam, TeamType& team);
void PrintRoster(StartingType& StartingTeam, TeamType& team);
void PrintTeam(TeamType& team);
bool Done( );
void ChooseStarters(StartingType& StartingTeam, TeamType& team);

//***************************************************************************
int main( )
{
   TeamType team(NumJerseys);
   StartingType StartingTeam(NumberOnTeam);
   Initialize(team);
   ReadFile(team);
   do {
      clrscr( );
      PrintTeam(team);
      ChooseStarters(StartingTeam,team);
      PrintRoster(StartingTeam,team);
   } while ( !Done( ) );
   cout << endl;
   cout << "This Team was chosen by -----YOUR NAME-----" << endl;
   getch( );
   return 0;
} 
//***************************************************************************
void Initialize (TeamType& team)
// pre : none
//post : exists member of team entries will be set to false
{
   for (int count=0; count<NumJerseys; count++)
      team[count].exists = false;
}
//***************************************************************************
void ReadFile (TeamType& team)
//  pre : the exist fields of team have been set to false
// post : information from one of the four files is read into team array
//        at location indicated by the jersey number
{
   int      number;
   apstring tempName;
   char     ch;
   ifstream infile;
   apstring filename;
   cout << "Enter the file name: ";
   cin  >> filename;     // one of the four varsity or JV basketball teams
   infile.open(filename.c_str());    
   if( !infile )
   {
      cerr<< "File not open successfully.\n";
      exit(1);
   }
   while ( infile >> tempName >> number >> ch )
   {
      team[number].name = tempName;
      switch (ch)
      {
         case 'c' : team[number].position = center; break;
         case 'f' : team[number].position = forward; break;
         case 'g' : team[number].position = guard;
      }
      infile >> team[number].height;     
      team[number].exists = true;        // player with this jersey # exists
   }
} 
//***************************************************************************
void PrintHeadings (bool All)
// pre : if All is true, heights will also be printed
{
   cout << setw(30) << "VARSITY TEAM ROSTER" << endl;
   cout << endl;
   cout << setw(8) << "NUMBER" << setw(10) << "NAME"
        << setw(20) << "POSITION";
   if ( All )
      cout << setw(15) << "HEIGHT" << endl;
   else cout << endl;
   cout << endl;
} 
//***************************************************************************
void PrintPlayer (int number, PlayerType& player, bool All)
// pre : number contains jersey number of player in range 0 to 55
//       team contains information about players
//       All is true if height is to be printed
// action : information about given player will be printed
{
   cout << setw(5) << number;
   cout << setw(8) << " " << player.name;
   switch (player.position)
   {
      case  guard : cout << setw(27-player.name.length()) << "guard  "; break;
      case forward: cout << setw(27-player.name.length()) << "forward"; break;
      case  center: cout << setw(27-player.name.length()) << "center ";
   }
   if ( All )
      cout << "         " << player.height << endl;
   else cout << endl;
} 
//***************************************************************************
bool Duplicates(StartingType& StartingTeam)
//  pre : StartingTeam contains the numbers for the starting team in range
// post : Duplicates is false if there are no duplicate numbers;
//        otherwise it is true
{
   int Dup = false;
   for (int i=0; i<NumberOnTeam-1; i++)
      for (int j=i+1; j<NumberOnTeam; j++)
        if (StartingTeam[i] == StartingTeam[j])
           Dup = true;
   return Dup;
} 
//***************************************************************************
bool Valid (StartingType& StartingTeam,TeamType& team)
// pre: StartingTeam contains the numbers for the starting line-up in range
//      0 to 55; team has information for members on the team
//post: valid will be true if the team has one center, two forwards
//      and two guards and there are not any duplicates; otherwise
//      valid will be false
{
   PositionType place;
   RosterType roster(PositionType(3));
   for (place=center; place<=guard; place=PositionType(place+1))
      roster[place] = 0;
   for (int i=0; i<NumberOnTeam; i++)
   {
      place = team[StartingTeam[i]].position;
      roster[place]++;
   }
   return ( roster[center]==1 && roster[guard]==2 &&
           roster[forward]==2 && !Duplicates(StartingTeam) );
} 
//***************************************************************************
void PrintRoster (StartingType& StartingTeam, TeamType& team)
// This procedure will print all the players on the starting team
// in order of player number
{
   if ( Valid(StartingTeam,team) )
   {
      PrintHeadings(false);
      for (int member = 0; member<NumberOnTeam; member++)
         PrintPlayer(StartingTeam[member],team[StartingTeam[member]],false);
   }
   else
   {
      cout << "Invalid starting line-up." << endl;
      cout << "You need one center, two forwards, and two guards." << endl;
   }
   cout << endl;
} 
//***************************************************************************
void PrintTeam (TeamType& team)
{
   PrintHeadings(true);
   for (int member=0; member<NumJerseys; member++)
      if (team[member].exists)
         PrintPlayer(member,team[member],true);
   cout << endl;
} 
//***************************************************************************
bool Done()
// pre : none
// post: done will be false if answer is yes; otherwise true
{
   cout << endl;
   cout << "Do you want to continue?";
   char answer;
   cin >> answer;
   return ( answer!='Y' && answer!='y' );
} 
//***************************************************************************
void ChooseStarters(StartingType& StartingTeam, TeamType& team)
// pre: team has been given information for team members
//post: StartingTeam contains the numbers chosen for the starting line-up
{
   cout << "Enter starting line-up." << endl;
   bool valid;
   for (int member=0; member<NumberOnTeam; member++)
   {
      do {
         cout << "Enter number for player number " << member+1 << " > ";
         valid = true;

         cin >> StartingTeam[member];
         if ( !(StartingTeam[member]>=0 && StartingTeam[member]<=NumJerseys) )
         {
            cout << "number must be in range 0 .. " << NumJerseys-1 << endl;
            valid = false;
         }
         if ( (valid) && (!team[StartingTeam[member]].exists) )
         {
            cout << error << endl;
            valid = false;
         }
      } while ( !valid);
   }
} 
//****************************************************************************
